searchNodes=[{"doc":"Flop is a helper library for filtering, ordering and pagination with Ecto. Usage The simplest way of using this library is just to use Flop.validate_and_run/3 and Flop.validate_and_run!/3 . Both functions take a queryable and a parameter map, validate the parameters, run the query and return the query results and the meta information. iex&gt; Flop.Repo . insert_all ( Flop.Pet , [ ...&gt; %{ name : &quot;Harry&quot; , age : 4 , species : &quot;C. lupus&quot; } , ...&gt; %{ name : &quot;Maggie&quot; , age : 1 , species : &quot;O. cuniculus&quot; } , ...&gt; %{ name : &quot;Patty&quot; , age : 2 , species : &quot;C. aegagrus&quot; } ...&gt; ] ) iex&gt; params = %{ order_by : [ &quot;name&quot; , &quot;age&quot; ] , page : 1 , page_size : 2 } iex&gt; { :ok , { results , meta } } = ...&gt; Flop . validate_and_run ( ...&gt; Flop.Pet , ...&gt; params , ...&gt; repo : Flop.Repo ...&gt; ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Harry&quot; , &quot;Maggie&quot; ] iex&gt; meta . total_count 3 iex&gt; meta . total_pages 2 iex&gt; meta . has_next_page? true Under the hood, these functions just call Flop.validate/2 and Flop.run/3 , which in turn calls Flop.all/3 and Flop.meta/3 . If you need finer control about if and when to execute each step, you can call those functions directly. See Flop.Meta for descriptions of the meta fields. Global configuration You can set some global options like the default Ecto repo via the application environment. All global options can be overridden by passing them directly to the functions or configuring the options for a schema module via Flop.Schema . import Config config :flop , repo : MyApp.Repo See Flop.option/0 for a description of all available options. Schema options You can set some options for a schema by deriving Flop.Schema . The options are evaluated at the validation step. defmodule Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , default_limit : 20 , max_limit : 100 } schema &quot;pets&quot; do field :name , :string field :age , :integer field :species , :string field :social_security_number , :string end end You need to pass the schema to Flop.validate/2 or any function that includes the validation step with the :for option. iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , &quot;limit&quot; =&gt; 5 } iex&gt; { :ok , flop } = Flop . validate ( params , for : Flop.Pet ) iex&gt; flop . limit 5 iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , &quot;limit&quot; =&gt; 10_000 } iex&gt; { :error , changeset } = Flop . validate ( params , for : Flop.Pet ) iex&gt; [ { :limit , { msg , _ } } ] = changeset . errors iex&gt; msg &quot;must be less than or equal to %{number}&quot; iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , &quot;limit&quot; =&gt; 10_000 } iex&gt; { :error , changeset } = ...&gt; Flop . validate_and_run ( ...&gt; Flop.Pet , ...&gt; params , ...&gt; for : Flop.Pet ...&gt; ) iex&gt; [ { :limit , { msg , _ } } ] = changeset . errors iex&gt; msg &quot;must be less than or equal to %{number}&quot; Ordering To add an ordering clause to a query, you need to set the :order_by and optionally the :order_directions parameter. :order_by should be the list of fields, while :order_directions is a list of Flop.order_direction/0 . :order_by and :order_directions are zipped when generating the ORDER BY clause. If no order directions are given, :asc is used as default. iex&gt; params = %{ ...&gt; &quot;order_by&quot; =&gt; [ &quot;name&quot; , &quot;age&quot; ] , ...&gt; &quot;order_directions&quot; =&gt; [ &quot;asc&quot; , &quot;desc&quot; ] ...&gt; } iex&gt; { :ok , flop } = Flop . validate ( params ) iex&gt; flop . order_by [ :name , :age ] iex&gt; flop . order_directions [ :asc , :desc ] Flop uses these two fields instead of a keyword list, so that the order instructions can be easily passed in a query string. Pagination For queries using OFFSET and LIMIT , you have the choice between page-based pagination parameters: %{ page : 5 , page_size : 20 } and offset-based pagination parameters: %{ offset : 100 , limit : 20 } For cursor-based pagination, you can either use :first / :after or :last / :before . You also need to pass the :order_by parameter or set a default order for the schema via Flop.Schema . iex&gt; Flop.Repo . insert_all ( Flop.Pet , [ ...&gt; %{ name : &quot;Harry&quot; , age : 4 , species : &quot;C. lupus&quot; } , ...&gt; %{ name : &quot;Maggie&quot; , age : 1 , species : &quot;O. cuniculus&quot; } , ...&gt; %{ name : &quot;Patty&quot; , age : 2 , species : &quot;C. aegagrus&quot; } ...&gt; ] ) iex&gt; iex&gt; # forward (first/after) iex&gt; iex&gt; params = %{ first : 2 , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( Flop.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Patty&quot; , &quot;Harry&quot; ] iex&gt; meta . has_next_page? true iex&gt; end_cursor = meta . end_cursor &quot;g3QAAAACZAAEbmFtZW0AAAAFSGFycnlkAAdzcGVjaWVzbQAAAAhDLiBsdXB1cw==&quot; iex&gt; params = %{ first : 2 , after : end_cursor , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( Flop.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Maggie&quot; ] iex&gt; meta . has_next_page? false iex&gt; iex&gt; # backward (last/before) iex&gt; iex&gt; params = %{ last : 2 , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( Flop.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Harry&quot; , &quot;Maggie&quot; ] iex&gt; meta . has_previous_page? true iex&gt; start_cursor = meta . start_cursor &quot;g3QAAAACZAAEbmFtZW0AAAAFSGFycnlkAAdzcGVjaWVzbQAAAAhDLiBsdXB1cw==&quot; iex&gt; params = %{ last : 2 , before : start_cursor , order_by : [ :species , :name ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( Flop.Pet , params ) iex&gt; Enum . map ( results , &amp; &amp;1 . name ) [ &quot;Patty&quot; ] iex&gt; meta . has_previous_page? false By default, it is assumed that the query result is a list of maps or structs. If your query returns a different data structure, you can pass the :cursor_value_func option to retrieve the cursor values. See Flop.option/0 and Flop.Cursor for more information. You can restrict which pagination types are available. See Flop.option/0 for details. Filters Filters can be passed as a list of maps. It is recommended to define the filterable fields for a schema using Flop.Schema . iex&gt; Flop.Repo . insert_all ( Flop.Pet , [ ...&gt; %{ name : &quot;Harry&quot; , age : 4 , species : &quot;C. lupus&quot; } , ...&gt; %{ name : &quot;Maggie&quot; , age : 1 , species : &quot;O. cuniculus&quot; } , ...&gt; %{ name : &quot;Patty&quot; , age : 2 , species : &quot;C. aegagrus&quot; } ...&gt; ] ) iex&gt; iex&gt; params = %{ filters : [ %{ field : :name , op : :=~ , value : &quot;Mag&quot; } ] } iex&gt; { :ok , { results , meta } } = Flop . validate_and_run ( Flop.Pet , params ) iex&gt; meta . total_count 1 iex&gt; [ pet ] = results iex&gt; pet . name &quot;Maggie&quot; See Flop.Filter.op/0 for a list of all available filter operators. GraphQL and Relay The parameters used for cursor-based pagination follow the Relay specification, so you can just pass the arguments you get from the client on to Flop. Flop.Relay can convert the query results returned by Flop.validate_and_run/3 into Edges and PageInfo formats required for Relay connections. For example, if you have a context module like this: defmodule MyApp.Flora import Ecto . query , warn : false alias MyApp.Flora.Plant def list_plants_by_continent ( % Continent { } = continent , %{ } = args ) do Plant |&gt; where ( continent_id : ^ continent . id ) |&gt; Flop . validate_and_run ( args , for : Plant ) end end Then your Absinthe resolver for the plants connection may look something like this: def list_plants ( args , %{ source : % Continent { } = continent } ) do with { :ok , result } &lt;- Flora . list_plants_by_continent ( continent , args ) do { :ok , Flop.Relay . connection_from_result ( result ) } end end","ref":"Flop.html","title":"Flop","type":"module"},{"doc":"Applies the given Flop to the given queryable and returns all matchings entries. iex&gt; Flop . all ( Flop.Pet , % Flop { } , repo : Flop.Repo ) [ ] You can also configure a default repo in your config files: config :flop , repo : MyApp.Repo This allows you to omit the third argument: iex&gt; Flop . all ( Flop.Pet , % Flop { } ) [ ] Note that when using cursor-based pagination, the applied limit will be first + 1 or last + 1 . The extra record is removed by Flop.run/3 , but not by this function.","ref":"Flop.html#all/3","title":"Flop.all/3","type":"function"},{"doc":"Returns the total count of entries matching the filter conditions of the Flop. The pagination and ordering option are disregarded. iex&gt; Flop . count ( Flop.Pet , % Flop { } , repo : Flop.Repo ) 0 You can also configure a default repo in your config files: config :flop , repo : MyApp.Repo This allows you to omit the third argument: iex&gt; Flop . count ( Flop.Pet , % Flop { } ) 0","ref":"Flop.html#count/3","title":"Flop.count/3","type":"function"},{"doc":"Applies the filter parameter of a Flop.t/0 to an Ecto.Queryable.t/0 . Used by Flop.query/2 .","ref":"Flop.html#filter/3","title":"Flop.filter/3","type":"function"},{"doc":"Returns the option with the given key. The look-up order is: the keyword list passed as the second argument the schema module that derives Flop.Schema , if the passed list includes the :for option the application environment","ref":"Flop.html#get_option/2","title":"Flop.get_option/2","type":"function"},{"doc":"Returns meta information for the given query and flop that can be used for building the pagination links. iex&gt; Flop . meta ( Flop.Pet , % Flop { limit : 10 } , repo : Flop.Repo ) % Flop.Meta { current_offset : 0 , current_page : 1 , end_cursor : nil , flop : % Flop { limit : 10 } , has_next_page? : false , has_previous_page? : false , next_offset : nil , next_page : nil , page_size : 10 , previous_offset : nil , previous_page : nil , start_cursor : nil , total_count : 0 , total_pages : 0 } The function returns both the current offset and the current page, regardless of the pagination type. If the offset lies in between pages, the current page number is rounded up. This means that it is possible that the values for current_page and next_page can be identical. This can only occur if you use offset/limit based pagination with arbitrary offsets, but in that case, you will use the previous_offset , current_offset and next_offset values to render the pagination links anyway, so this shouldn't be a problem. Unless cursor-based pagination is used, this function will run a query to figure get the total count of matching records.","ref":"Flop.html#meta/3","title":"Flop.meta/3","type":"function"},{"doc":"Applies the order_by and order_directions parameters of a Flop.t/0 to an Ecto.Queryable.t/0 . Used by Flop.query/2 .","ref":"Flop.html#order_by/3","title":"Flop.order_by/3","type":"function"},{"doc":"Applies the pagination parameters of a Flop.t/0 to an Ecto.Queryable.t/0 . The function supports both offset / limit based pagination and page / page_size based pagination. If you validated the Flop.t/0 with Flop.validate/1 before, you can be sure that the given Flop.t/0 only has pagination parameters set for one pagination method. If you pass an unvalidated Flop.t/0 that has pagination parameters set for multiple pagination methods, this function will arbitrarily only apply one of the pagination methods. Used by Flop.query/2 .","ref":"Flop.html#paginate/2","title":"Flop.paginate/2","type":"function"},{"doc":"Updates the order_by and order_directions values of a Flop struct. If the field is not in the current order_by value, it will be prepended to the list. The order direction for the field will be set to :asc . If the field is already at the front of the order_by list, the order direction will be reversed. If the field is already in the list, but not at the front, it will be moved to the front and the order direction will be set to :asc . Example iex&gt; flop = push_order ( % Flop { } , :name ) iex&gt; flop . order_by [ :name ] iex&gt; flop . order_directions [ :asc ] iex&gt; flop = push_order ( flop , :age ) iex&gt; flop . order_by [ :age , :name ] iex&gt; flop . order_directions [ :asc , :asc ] iex&gt; flop = push_order ( flop , :age ) iex&gt; flop . order_by [ :age , :name ] iex&gt; flop . order_directions [ :desc , :asc ] iex&gt; flop = push_order ( flop , :species ) iex&gt; flop . order_by [ :species , :age , :name ] iex&gt; flop . order_directions [ :asc , :desc , :asc ] iex&gt; flop = push_order ( flop , :age ) iex&gt; flop . order_by [ :age , :species , :name ] iex&gt; flop . order_directions [ :asc , :asc , :asc ] If a string is passed as the second argument, it will be converted to an atom using String.to_existing_atom/1 . If the atom does not exist, the Flop struct will be returned unchanged. iex&gt; flop = push_order ( % Flop { } , &quot;name&quot; ) iex&gt; flop . order_by [ :name ] iex&gt; flop = push_order ( % Flop { } , &quot;this_atom_does_not_exist&quot; ) iex&gt; flop . order_by nil","ref":"Flop.html#push_order/2","title":"Flop.push_order/2","type":"function"},{"doc":"Adds clauses for filtering, ordering and pagination to a Ecto.Queryable.t/0 . The parameters are represented by the Flop.t/0 type. Any nil values will be ignored. Examples iex&gt; flop = % Flop { limit : 10 , offset : 19 } iex&gt; Flop . query ( Flop.Pet , flop ) # Ecto.Query &lt; from p0 in Flop.Pet , limit : ^ 10 , offset : ^ 19 &gt; Or enhance an already defined query: iex&gt; require Ecto.Query iex&gt; flop = % Flop { limit : 10 } iex&gt; Flop.Pet |&gt; Ecto.Query . where ( species : &quot;dog&quot; ) |&gt; Flop . query ( flop ) # Ecto.Query &lt; from p0 in Flop.Pet , where : p0 . species == &quot;dog&quot; , limit : ^ 10 &gt; Note that when using cursor-based pagination, the applied limit will be first + 1 or last + 1 . The extra record is removed by Flop.run/3 .","ref":"Flop.html#query/3","title":"Flop.query/3","type":"function"},{"doc":"Applies the given Flop to the given queryable, retrieves the data and the meta data. This function does not validate the given flop parameters. You can validate the parameters with Flop.validate/2 or Flop.validate!/2 , or you can use Flop.validate_and_run/3 or Flop.validate_and_run!/3 instead of this function. iex&gt; { data , meta } = Flop . run ( Flop.Pet , % Flop { } ) iex&gt; data == [ ] true iex&gt; match? ( % Flop.Meta { } , meta ) true","ref":"Flop.html#run/3","title":"Flop.run/3","type":"function"},{"doc":"Sets the page value of a Flop struct while also removing/converting pagination parameters for other pagination types. iex&gt; set_page ( % Flop { page : 2 , page_size : 10 } , 6 ) % Flop { page : 6 , page_size : 10 } iex&gt; set_page ( % Flop { limit : 10 , offset : 20 } , 8 ) % Flop { limit : nil , offset : nil , page : 8 , page_size : 10 } The page number will not be allowed to go below 1. iex&gt; set_page ( % Flop { } , - 5 ) % Flop { page : 1 }","ref":"Flop.html#set_page/2","title":"Flop.set_page/2","type":"function"},{"doc":"Validates a Flop.t/0 . Examples iex&gt; params = %{ &quot;limit&quot; =&gt; 10 , &quot;offset&quot; =&gt; 0 , &quot;texture&quot; =&gt; &quot;fluffy&quot; } iex&gt; Flop . validate ( params ) { :ok , % Flop { filters : [ ] , limit : 10 , offset : 0 , order_by : nil , order_directions : nil , page : nil , page_size : nil } } iex&gt; flop = % Flop { offset : - 1 } iex&gt; { :error , changeset } = Flop . validate ( flop ) iex&gt; changeset . valid? false iex&gt; changeset . errors [ offset : { &quot;must be greater than or equal to %{number}&quot; , [ validation : :number , kind : :greater_than_or_equal_to , number : 0 ] } ] It also makes sure that only one pagination method is used. iex&gt; params = %{ limit : 10 , offset : 0 , page : 5 , page_size : 10 } iex&gt; { :error , changeset } = Flop . validate ( params ) iex&gt; changeset . valid? false iex&gt; changeset . errors [ limit : { &quot;cannot combine multiple pagination types&quot; , [ ] } ] If you derived Flop.Schema in your Ecto schema to define the filterable and sortable fields, you can pass the module name to the function to validate that only allowed fields are used. The function will also apply any default values set for the schema. iex&gt; params = %{ &quot;order_by&quot; =&gt; [ &quot;species&quot; ] } iex&gt; { :error , changeset } = Flop . validate ( params , for : Flop.Pet ) iex&gt; changeset . valid? false iex&gt; [ order_by : { msg , [ _ , { _ , enum } ] } ] = changeset . errors iex&gt; msg &quot;has an invalid entry&quot; iex&gt; enum [ :name , :age ] Note that currently, trying to use an existing field that is not allowed as seen above will result in the error message has an invalid entry , while trying to use a field name that does not exist in the schema (or more precisely: a field name that doesn't exist as an atom) will result in the error message is invalid . This might change in the future.","ref":"Flop.html#validate/2","title":"Flop.validate/2","type":"function"},{"doc":"Same as Flop.validate/2 , but raises an Ecto.InvalidChangesetError if the parameters are invalid.","ref":"Flop.html#validate!/2","title":"Flop.validate!/2","type":"function"},{"doc":"Validates the given flop parameters and retrieves the data and meta data on success. iex&gt; { :ok , { [ ] , % Flop.Meta { } } } = ...&gt; Flop . validate_and_run ( Flop.Pet , % Flop { } , for : Flop.Pet ) iex&gt; { :error , % Ecto.Changeset { } = changeset } = ...&gt; Flop . validate_and_run ( Flop.Pet , % Flop { limit : - 1 } ) iex&gt; changeset . errors [ limit : { &quot;must be greater than %{number}&quot; , [ validation : :number , kind : :greater_than , number : 0 ] } ] Options for : Passed to Flop.validate/2 . repo : The Ecto.Repo module. Required if no default repo is configured. cursor_value_func : An arity-2 function to be used to retrieve an unencoded cursor value from a query result item and the order_by fields. Defaults to Flop.Cursor.get_cursor_from_node/2 .","ref":"Flop.html#validate_and_run/3","title":"Flop.validate_and_run/3","type":"function"},{"doc":"Same as Flop.validate_and_run/3 , but raises on error.","ref":"Flop.html#validate_and_run!/3","title":"Flop.validate_and_run!/3","type":"function"},{"doc":"Options that can be passed to most of the functions or that can be set via the application environment. :for - The schema module to be used for validation. Flop.Schema must be derived for the given module. This option is optional and can not be set globally. If it is not set, schema specific validation will be omitted. Used by the validation functions and passed on by any function calling a validation function. :default_limit - Sets a global default limit for queries that is used if no default limit is set for a schema and no limit is set in the parameters. Can only be set in the application configuration. :filtering (boolean) - Can be set to false to silently ignore filter parameters. :cursor_value_func - 2-arity function used to get the (unencoded) cursor value from a record. Only used with cursor-based pagination. The first argument is the record, the second argument is the list of fields used in the ORDER BY clause. Needs to return a map with the order fields as keys and the the record values of these fields as values. Defaults to Flop.Cursor.get_cursor_from_node/2 . :max_limit - Sets a global maximum limit for queries that is used if no maximum limit is set for a schema. Can only be set in the application configuration. :pagination_types - Defines which pagination types are allowed. Passing parameters for other pagination types will result in a validation error. By default, all pagination types are allowed. See also Flop.pagination_type/0 . Note that an offset value of 0 and a limit are still accepted even if offset-based pagination is disabled. :ordering (boolean) - Can be set to false to silently ignore order parameters. Default orders are still applied. :prefix - Configures the query to be executed with the given query prefix. See the Ecto documentation on &quot;Query prefix&quot; . :repo - The Ecto Repo module to use for the database query. Used by all functions that execute a database query. All options can be passed directly to the functions. Some of the options can be set on a schema level via Flop.Schema . All options except :for can be set globally via the application environment. import Config config :flop , default_limit : 25 , filtering : false , cursor_value_func : &amp; MyApp.Repo . get_cursor_value / 2 , max_limit : 100 , ordering : false , pagination_types : [ :first , :last , :page ] , repo : MyApp.Repo , prefix : &quot;some-prefix&quot; The look up order is: option passed to function option set for schema using Flop.Schema (only :max_limit , :default_limit and :pagination_types ) option set in global config (except :for ) default value (only :cursor_value_func )","ref":"Flop.html#t:option/0","title":"Flop.option/0","type":"type"},{"doc":"Represents the supported order direction values.","ref":"Flop.html#t:order_direction/0","title":"Flop.order_direction/0","type":"type"},{"doc":"Represents the pagination type. :offset - pagination using the offset and limit parameters :page - pagination using the page and page_size parameters :first - cursor-based pagination using the first and after parameters :last - cursor-based pagination using the last and before parameters","ref":"Flop.html#t:pagination_type/0","title":"Flop.pagination_type/0","type":"type"},{"doc":"Represents the query parameters for filtering, ordering and pagination. Fields after : Used for cursor-based pagination. Must be used with first or a default limit. before : Used for cursor-based pagination. Must be used with last or a default limit. limit , offset : Used for offset-based pagination. first Used for cursor-based pagination. Can be used alone to begin pagination or with after last Used for cursor-based pagination. page , page_size : Used for offset-based pagination as an alternative to offset and limit . order_by : List of fields to order by. Fields can be restricted by deriving Flop.Schema in your Ecto schema. order_directions : List of order directions applied to the fields defined in order_by . If empty or the list is shorter than the order_by list, :asc will be used as a default for each missing order direction. filters : List of filters, see Flop.Filter.t/0 .","ref":"Flop.html#t:t/0","title":"Flop.t/0","type":"type"},{"doc":"Functions for encoding, decoding and extracting cursor values.","ref":"Flop.Cursor.html","title":"Flop.Cursor","type":"module"},{"doc":"Decodes a cursor value. Returns :error if the cursor cannot be decoded or the decoded term is not a map with atom keys. iex&gt; Flop.Cursor . decode ( &quot;g3QAAAABZAACaWRiAAACDg==&quot; ) { :ok , %{ id : 526 } } iex&gt; Flop.Cursor . decode ( &quot;AAAH&quot; ) :error iex&gt; f = fn a -&gt; a + 1 end iex&gt; cursor = Flop.Cursor . encode ( %{ a : f } ) iex&gt; Flop.Cursor . decode ( cursor ) :error iex&gt; cursor = Flop.Cursor . encode ( a : &quot;b&quot; ) iex&gt; Flop.Cursor . decode ( cursor ) :error iex&gt; cursor = Flop.Cursor . encode ( %{ &quot;a&quot; =&gt; &quot;b&quot; } ) iex&gt; Flop.Cursor . decode ( cursor ) :error Trying to decode a cursor that contains non-existent atoms also results in an error. iex&gt; Flop.Cursor . decode ( &quot;g3QAAAABZAAGYmFybmV5ZAAGcnViYmVs&quot; ) :error","ref":"Flop.Cursor.html#decode/1","title":"Flop.Cursor.decode/1","type":"function"},{"doc":"Same as Flop.Cursor.decode/1 , but raises an error if the cursor is invalid. iex&gt; Flop.Cursor . decode! ( &quot;g3QAAAABZAACaWRiAAACDg==&quot; ) %{ id : 526 } iex&gt; Flop.Cursor . decode! ( &quot;AAAH&quot; ) ** (RuntimeError) invalid cursor","ref":"Flop.Cursor.html#decode!/1","title":"Flop.Cursor.decode!/1","type":"function"},{"doc":"Encodes a cursor value. iex&gt; Flop.Cursor . encode ( %{ name : &quot;Peter&quot; , email : &quot;peter@mail&quot; } ) &quot;g3QAAAACZAAFZW1haWxtAAAACnBldGVyQG1haWxkAARuYW1lbQAAAAVQZXRlcg==&quot;","ref":"Flop.Cursor.html#encode/1","title":"Flop.Cursor.encode/1","type":"function"},{"doc":"Takes a tuple with the node and the edge and the order_by field list and returns the cursor value derived from the edge map. If a map is passed instead of a tuple, it retrieves the cursor value from that map. This function can be used for the :cursor_value_func option. See also Flop.Cursor.get_cursor_from_node/2 . iex&gt; record = %{ id : 20 , name : &quot;George&quot; , age : 62 } iex&gt; edge = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; iex&gt; Flop.Cursor . get_cursor_from_edge ( { record , edge } , [ :id ] ) %{ id : 25 } iex&gt; Flop.Cursor . get_cursor_from_edge ( { record , edge } , [ :id , :relation ] ) %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; Flop.Cursor . get_cursor_from_edge ( record , [ :id ] ) %{ id : 20 }","ref":"Flop.Cursor.html#get_cursor_from_edge/2","title":"Flop.Cursor.get_cursor_from_edge/2","type":"function"},{"doc":"Takes a map or a struct and the order_by field list and returns the cursor value. See also Flop.Cursor.get_cursor_from_edge/2 and Flop.Cursor.get_cursor_from_node/2 . iex&gt; record = %{ id : 20 , name : &quot;George&quot; , age : 62 } iex&gt; iex&gt; Flop.Cursor . get_cursor_from_map ( record , [ :id ] ) %{ id : 20 } iex&gt; Flop.Cursor . get_cursor_from_map ( record , [ :name , :age ] ) %{ age : 62 , name : &quot;George&quot; }","ref":"Flop.Cursor.html#get_cursor_from_map/2","title":"Flop.Cursor.get_cursor_from_map/2","type":"function"},{"doc":"Takes a tuple with the node and the edge and the order_by field list and returns the cursor value derived from the node map. If a map is passed instead of a tuple, it retrieves the cursor value from that map. This function is used as a default if no :cursor_value_func option is set. See also Flop.Cursor.get_cursor_from_edge/2 . iex&gt; record = %{ id : 20 , name : &quot;George&quot; , age : 62 } iex&gt; edge = %{ id : 25 , relation : &quot;sibling&quot; } iex&gt; iex&gt; Flop.Cursor . get_cursor_from_node ( { record , edge } , [ :id ] ) %{ id : 20 } iex&gt; Flop.Cursor . get_cursor_from_node ( { record , edge } , [ :id , :name ] ) %{ id : 20 , name : &quot;George&quot; } iex&gt; Flop.Cursor . get_cursor_from_node ( record , [ :id ] ) %{ id : 20 }","ref":"Flop.Cursor.html#get_cursor_from_node/2","title":"Flop.Cursor.get_cursor_from_node/2","type":"function"},{"doc":"Retrieves the start and end cursors from a query result. iex&gt; results = [ %{ name : &quot;Mary&quot; } , %{ name : &quot;Paul&quot; } , %{ name : &quot;Peter&quot; } ] iex&gt; order_by = [ :name ] iex&gt; iex&gt; { start_cursor , end_cursor } = ...&gt; Flop.Cursor . get_cursors ( results , order_by ) { &quot;g3QAAAABZAAEbmFtZW0AAAAETWFyeQ==&quot; , &quot;g3QAAAABZAAEbmFtZW0AAAAFUGV0ZXI=&quot; } iex&gt; iex&gt; Flop.Cursor . decode ( start_cursor ) { :ok , %{ name : &quot;Mary&quot; } } iex&gt; Flop.Cursor . decode ( end_cursor ) { :ok , %{ name : &quot;Peter&quot; } } If the result set is empty, the cursor values will be nil . iex&gt; Flop.Cursor . get_cursors ( [ ] , [ :id ] ) { nil , nil } The default function to retrieve the cursor value from the query result is Flop.Cursor.get_cursor_from_node/2 , which expects the query result to be a map or a 2-tuple. You can set the cursor_value_func option to use another function. Flop also comes with Flop.Cursor.get_cursor_from_edge/2 . If the records in the result set are not maps, you can define a custom cursor value function like this: iex&gt; results = [ { &quot;Mary&quot; , 1936 } , { &quot;Paul&quot; , 1937 } , { &quot;Peter&quot; , 1938 } ] iex&gt; cursor_func = fn { name , year } , order_fields -&gt; ...&gt; Enum . into ( order_fields , %{ } , fn ...&gt; :name -&gt; { :name , name } ...&gt; :year -&gt; { :year , year } ...&gt; end ) ...&gt; end iex&gt; opts = [ cursor_value_func : cursor_func ] iex&gt; iex&gt; { start_cursor , end_cursor } = ...&gt; Flop.Cursor . get_cursors ( results , [ :name , :year ] , opts ) { &quot;g3QAAAACZAAEbmFtZW0AAAAETWFyeWQABHllYXJiAAAHkA==&quot; , &quot;g3QAAAACZAAEbmFtZW0AAAAFUGV0ZXJkAAR5ZWFyYgAAB5I=&quot; } iex&gt; iex&gt; Flop.Cursor . decode ( start_cursor ) { :ok , %{ name : &quot;Mary&quot; , year : 1936 } } iex&gt; Flop.Cursor . decode ( end_cursor ) { :ok , %{ name : &quot;Peter&quot; , year : 1938 } }","ref":"Flop.Cursor.html#get_cursors/3","title":"Flop.Cursor.get_cursors/3","type":"function"},{"doc":"Defines a filter.","ref":"Flop.Filter.html","title":"Flop.Filter","type":"module"},{"doc":"Represents valid filter operators. Operator Value WHERE clause :== &quot;Salicaceae&quot; WHERE column = 'Salicaceae' :!= &quot;Salicaceae&quot; WHERE column != 'Salicaceae' :=~ &quot;cyth&quot; WHERE column ILIKE '%cyth%' :empty WHERE column IS NULL :not_empty WHERE column IS NOT NULL :&lt;= 10 WHERE column &lt;= 10 :&lt; 10 WHERE column &lt; 10 :&gt;= 10 WHERE column &gt;= 10 :&gt; 10 WHERE column &gt; 10 :in [&quot;pear&quot;, &quot;plum&quot;] WHERE column IN ('pear', 'plum') :like &quot;cyth&quot; WHERE column LIKE '%cyth%' :like_and &quot;Rubi Rosa&quot; WHERE column LIKE '%Rubi%' AND column LIKE '%Rosa%' :like_or &quot;Rubi Rosa&quot; WHERE column LIKE '%Rubi%' OR column LIKE '%Rosa%' :ilike &quot;cyth&quot; WHERE column ILIKE '%cyth%' :ilike_and &quot;Rubi Rosa&quot; WHERE column ILIKE '%Rubi%' AND column ILIKE '%Rosa%' :ilike_or &quot;Rubi Rosa&quot; WHERE column ILIKE '%Rubi%' OR column ILIKE '%Rosa%'","ref":"Flop.Filter.html#t:op/0","title":"Flop.Filter.op/0","type":"type"},{"doc":"Represents filter query parameters. Fields field : The field the filter is applied to. The allowed fields can be restricted by deriving Flop.Schema in your Ecto schema. op : The filter operator. value : The comparison value of the filter.","ref":"Flop.Filter.html#t:t/0","title":"Flop.Filter.t/0","type":"type"},{"doc":"Defines a struct for holding meta information of a query result.","ref":"Flop.Meta.html","title":"Flop.Meta","type":"module"},{"doc":"Meta information for a query result. :flop - The Flop struct used in the query. :current_offset - The :offset value used in the query when using offset-based pagination or a derived value when using page-based pagination. Always nil when using cursor-based pagination. :current_page - The :page value used in the query when using page-based pagination or a derived value when using offset-based pagination. Note that the value will be rounded if the offset lies between pages. Always nil when using cursor-based pagination. :previous_offset , :next_offset , :previous_page , :next_page - Values based on :current_page and :current_offset / page_size . Always nil when using cursor-based pagination. :start_cursor , :end_cursor - The cursors of the first and last record in the result set. Only set when using cursor-based pagination with :first / :after or :last / :before . :has_previous_page? , :has_next_page? - Set in all pagination types. Note that :has_previous_page? is always false when using cursor-based pagination with :first / after and :has_next_page? is always false when using cursor-based pagination with :last / :before . :page_size - The page size or limit of the query. Set to the :first or :last parameter when using cursor-based pagination. :total_count - The total count of records for the given query. Always nil when using cursor-based pagination. :total_pages - The total page count based on the total record count and the page size. Always nil when using cursor-based pagination.","ref":"Flop.Meta.html#t:t/0","title":"Flop.Meta.t/0","type":"type"},{"doc":"Helpers to turn query results into Relay formats.","ref":"Flop.Relay.html","title":"Flop.Relay","type":"module"},{"doc":"Takes the query results returned by Flop.run/3 , Flop.validate_and_run/3 or Flop.validate_and_run!/3 and turns them into the Relay connection format. Example iex&gt; flop = % Flop { order_by : [ :name ] } iex&gt; meta = % Flop.Meta { flop : flop , start_cursor : &quot;a&quot; , end_cursor : &quot;b&quot; } iex&gt; result = { [ % Flop.Fruit { name : &quot;Apple&quot; , family : &quot;Rosaceae&quot; } ] , meta } iex&gt; Flop.Relay . connection_from_result ( result ) %{ edges : [ %{ cursor : &quot;g3QAAAABZAAEbmFtZW0AAAAFQXBwbGU=&quot; , node : % Flop.Fruit { family : &quot;Rosaceae&quot; , id : nil , name : &quot;Apple&quot; } } ] , page_info : %{ end_cursor : &quot;b&quot; , has_next_page : false , has_previous_page : false , start_cursor : &quot;a&quot; } } See Flop.Relay.edges_from_result/2 for an example of adding additional fields to the edge. Options :cursor_value_func : 2-arity function that takes an item from the query result and the order_by fields and returns the unencoded cursor value.","ref":"Flop.Relay.html#connection_from_result/2","title":"Flop.Relay.connection_from_result/2","type":"function"},{"doc":"Turns a list of query results into Relay edges. Simple queries If your query returns a list of maps or structs, the function will return the a list of edges with :cursor and :node as only fields. iex&gt; flop = % Flop { order_by : [ :name ] } iex&gt; meta = % Flop.Meta { flop : flop } iex&gt; result = { [ % Flop.Fruit { name : &quot;Apple&quot; , family : &quot;Rosaceae&quot; } ] , meta } iex&gt; Flop.Relay . edges_from_result ( result ) [ %{ cursor : &quot;g3QAAAABZAAEbmFtZW0AAAAFQXBwbGU=&quot; , node : % Flop.Fruit { name : &quot;Apple&quot; , family : &quot;Rosaceae&quot; } } ] Supplying additional edge information If the query result is a list of 2-tuples, this is interpreted as a tuple of the node information and the edge information. For example, if you have a query like this: Group |&gt; where ( [ g ] , g . id == ^ group_id ) |&gt; join ( :left , [ g ] , m in assoc ( g , :members ) ) |&gt; select ( [ g , m ] , { m , map ( m , [ :role ] ) } ) Then your query result looks something like: [ { % Member { id : 242 , name : &quot;Carl&quot; } , %{ role : :owner } } ] In this case, the members are the nodes, and the maps with the roles is seen as edge information. [ %{ cursor : &quot;AE98RNSTNGN&quot; , node : % Member { id : 242 , name : &quot;Carl&quot; } , role : :owner } ] Note that in this case, the whole tuple will be passed to the cursor value function, so that the cursor can be based on both node and edge fields. Here's an example with fruit that overrides the cursor value function: iex&gt; flop = % Flop { order_by : [ :name ] } iex&gt; meta = % Flop.Meta { flop : flop } iex&gt; items = [ { % Flop.Fruit { name : &quot;Apple&quot; } , %{ preparation : :grated } } ] iex&gt; func = fn { fruit , _edge } , order_by -&gt; Map . take ( fruit , order_by ) end iex&gt; Flop.Relay . edges_from_result ( ...&gt; { items , meta } , ...&gt; cursor_value_func : func ...&gt; ) [ %{ cursor : &quot;g3QAAAABZAAEbmFtZW0AAAAFQXBwbGU=&quot; , node : % Flop.Fruit { name : &quot;Apple&quot; } , preparation : :grated } ] Options :cursor_value_func : 2-arity function that takes an item from the query result and the order_by fields and returns the unencoded cursor value.","ref":"Flop.Relay.html#edges_from_result/2","title":"Flop.Relay.edges_from_result/2","type":"function"},{"doc":"Takes a Flop.Meta struct and returns a map with the Relay page info. Example iex&gt; Flop.Relay . page_info_from_meta ( % Flop.Meta { ...&gt; has_previous_page? : true , ...&gt; has_next_page? : true , ...&gt; start_cursor : &quot;a&quot; , ...&gt; end_cursor : &quot;b&quot; ...&gt; } ) %{ has_previous_page : true , has_next_page : true , start_cursor : &quot;a&quot; , end_cursor : &quot;b&quot; }","ref":"Flop.Relay.html#page_info_from_meta/1","title":"Flop.Relay.page_info_from_meta/1","type":"function"},{"doc":"","ref":"Flop.Relay.html#t:connection/0","title":"Flop.Relay.connection/0","type":"type"},{"doc":"","ref":"Flop.Relay.html#t:edge/0","title":"Flop.Relay.edge/0","type":"type"},{"doc":"","ref":"Flop.Relay.html#t:page_info/0","title":"Flop.Relay.page_info/0","type":"type"},{"doc":"This protocol allows you to set query options in your Ecto schemas. Usage Derive Flop.Schema in your Ecto schema and set the filterable and sortable fields. defmodule Flop.Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] } schema &quot;pets&quot; do field :name , :string field :age , :integer field :species , :string end end After that, you can pass the module as the :for option to Flop.validate/2 . iex&gt; Flop . validate ( % Flop { order_by : [ :name ] } , for : Flop.Pet ) { :ok , % Flop { filters : [ ] , limit : nil , offset : nil , order_by : [ :name ] , order_directions : nil , page : nil , page_size : nil } } iex&gt; { :error , changeset } = Flop . validate ( ...&gt; % Flop { order_by : [ :species ] } , for : Flop.Pet ...&gt; ) iex&gt; changeset . valid? false iex&gt; changeset . errors [ order_by : { &quot;has an invalid entry&quot; , [ validation : :subset , enum : [ :name , :age ] ] } ] Default and maximum limits To define a default or maximum limit, you can set the default_limit and max_limit option when deriving Flop.Schema . The maximum limit will be validated and the default limit applied by Flop.validate/1 . @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , max_limit : 100 , default_limit : 50 } Default sort order To define a default sort order, you can set the default_order_by and default_order_directions options when deriving Flop.Schema . The default values are applied by Flop.validate/1 . If no order directions are set, :asc is assumed for all fields. @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , default_order_by : [ :name , :age ] , default_order_directions : [ :asc , :desc ] } Restricting pagination types By default, page / page_size , offset / limit and cursor-based pagination ( first / after and last / before ) are enabled. If you want to restrict the pagination type for a schema, you can do that by setting the pagination_types option. @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age ] , pagination_types : [ :first , :last ] } See also Flop.option/0 and Flop.pagination_type/0 . Setting the value to nil allows all pagination types. Compound fields Sometimes you might need to apply a search term to multiple fields at once, e.g. you might want to search in both the family name and given name field. You can do that with Flop by defining a compound field. @derive { Flop.Schema , filterable : [ :full_name ] , sortable : [ ] , compound_fields : [ full_name : [ :family_name , :given_name ] ] } This allows you to use the field name :full_name as any other field in the filters. params = %{ filters : [ %{ field : :full_name , op : :== , value : &quot;margo&quot; } ] } This would translate to: WHERE family_name = &#39;margo&#39; OR given_name = &#39;martindale&#39; Partial matches and splitting of the search term can be achieved with one of the ilike operators. params = %{ filters : [ %{ field : :full_name , op : :ilike_and , value : &quot;margo martindale&quot; } ] } This would translate to: WHERE ( family_name ilike &#39;%margo%&#39; OR given_name = &#39;%margo%&#39; ) AND ( family_name ilike &#39;%martindale%&#39; OR given_name = &#39;%martindale%&#39; ) Filter operator rules :=~ , :like , :like_and , :like_or , :ilike , :ilike_and , :ilike_or - The filter value is split at whitespace characters as usual. The filter matches for a value if it matches for any of the fields. :empty - Matches if all fields of the compound field are nil . :not_empty - Matches if any field of the compound field is not nil . :== , :!= , :&lt;= , :&lt; , :&gt;= , :&gt; , :in - The filter value is normalized by splitting the string at whitespaces and joining it with a space. The values of all fields of the compound field are split by whitespace character and joined with a space, and the resulting values are joined with a space again. This will be added in a future version. These filter operators are ignored for compound fields at the moment. Join fields If you need filter or order across tables, you can define join fields. Note: Support for ordering by join fields will be added in a future version. As an example, let's define these schemas: schema &quot;owners&quot; do field :name , :string field :email , :string has_many :pets , Pet end schema &quot;pets&quot; do field :name , :string field :species , :string belongs_to :owner , Owner end And now we want to find all owners that have pets of the species &quot;E. africanus&quot; . To do this, first we need to define a join field on the Owner schema. @derive { Flop.Schema , filterable : [ :pet_species ] , sortable : [ :pet_species ] , join_fields : [ pet_species : { :pets , :species } ] } In this case, :pet_species would be the alias of the field that you can refer to in the filter and order parameters. In {:pets, :species} , :pets refers to a named binding and :species refers to a field in that binding. After setting up the join fields, you can write a query like this: params = %{ filters : [ %{ field : :pet_species , op : :== , value : &quot;E. africanus&quot; } ] } Owner |&gt; join ( :left , [ o ] , p in assoc ( o , :pets ) , as : :pets ) |&gt; Flop . validate_and_run! ( params , for : Owner ) Note that Flop doesn't create the join clauses for you. The named bindings already have to be present in the query you pass to the Flop functions.","ref":"Flop.Schema.html","title":"Flop.Schema","type":"protocol"},{"doc":"Returns the default limit of a schema. iex&gt; Flop.Schema . default_limit ( % Flop.Fruit { } ) 50","ref":"Flop.Schema.html#default_limit/1","title":"Flop.Schema.default_limit/1","type":"function"},{"doc":"Returns the default order of a schema. iex&gt; Flop.Schema . default_order ( % Flop.Fruit { } ) %{ order_by : [ :name ] , order_directions : [ :asc ] }","ref":"Flop.Schema.html#default_order/1","title":"Flop.Schema.default_order/1","type":"function"},{"doc":"Returns the field type in a schema. {:normal, atom} - An ordinary field on the schema. The second tuple element is the field name. {:compound, [atom]} - A combination of fields defined with the compound_fields option. The list of atoms refers to the list of fields that are included. {:join, {atom, atom}} - A field from a named binding as defined with the join_fields option. The first atom refers to the binding name, the second atom refers to the field.","ref":"Flop.Schema.html#field_type/2","title":"Flop.Schema.field_type/2","type":"function"},{"doc":"Returns the filterable fields of a schema. iex&gt; Flop.Schema . filterable ( % Flop.Pet { } ) [ :age , :full_name , :name , :owner_age , :owner_name , :pet_and_owner_name , :species ]","ref":"Flop.Schema.html#filterable/1","title":"Flop.Schema.filterable/1","type":"function"},{"doc":"Returns the maximum limit of a schema. iex&gt; Flop.Schema . max_limit ( % Flop.Pet { } ) 1000","ref":"Flop.Schema.html#max_limit/1","title":"Flop.Schema.max_limit/1","type":"function"},{"doc":"Returns the allowed pagination types of a schema. iex&gt; Flop.Schema . pagination_types ( % Flop.Fruit { } ) [ :first , :last , :offset ]","ref":"Flop.Schema.html#pagination_types/1","title":"Flop.Schema.pagination_types/1","type":"function"},{"doc":"Returns the sortable fields of a schema. iex&gt; Flop.Schema . sortable ( % Flop.Pet { } ) [ :name , :age ]","ref":"Flop.Schema.html#sortable/1","title":"Flop.Schema.sortable/1","type":"function"},{"doc":"","ref":"Flop.Schema.html#t:t/0","title":"Flop.Schema.t/0","type":"type"},{"doc":"Flop Flop is an Elixir library that applies filtering, ordering and pagination parameters to your Ecto queries.","ref":"readme.html","title":"Flop","type":"extras"},{"doc":"offset-based pagination with offset / limit or page / page_size cursor-based pagination (aka key set pagination), compatible with Relay pagination arguments ordering by multiple fields in multiple directions filtering by multiple conditions with various operators on multiple fields parameter validation configurable filterable and sortable fields join fields compound fields query and meta data helpers Relay connection formatter (edges, nodes and page info) UI helpers and URL builders through Flop Phoenix .","ref":"readme.html#features","title":"Flop - Features","type":"extras"},{"doc":"This library has been used in production for a while now. Nevertheless, there may still be API changes on the way to a more complete feature set.","ref":"readme.html#status","title":"Flop - Status","type":"extras"},{"doc":"Add flop to your list of dependencies in mix.exs : def deps do [ { :flop , &quot;~&gt; 0.12.0&quot; } ] end If you want to configure a default repo, add this to your config file: config :flop , repo : MyApp.Repo","ref":"readme.html#installation","title":"Flop - Installation","type":"extras"},{"doc":"Define sortable and filterable fields To configure the sortable and filterable fields, derive Flop.Schema in your Ecto schema. While this step is optional, it is highly recommend, since the parameters you will pass to the Flop functions will come from the user side and should be validated. Deriving Flop.Schema will ensure that Flop only applies filtering and sorting parameters on the configured fields. defmodule MyApp.Pet do use Ecto.Schema @derive { Flop.Schema , filterable : [ :name , :species ] , sortable : [ :name , :age , :species ] } schema &quot;pets&quot; do field :name , :string field :age , :integer field :species , :string field :social_security_number , :string end end You can also define join fields, compound fields, max and default limit, and more. See the Flop.Schema documentation for all the options. Query data You can use Flop.validate_and_run/3 or Flop.validate_and_run!/3 to validate the Flop parameters, retrieve the data from the database and get the meta data for pagination in one go. defmodule MyApp.Pets do import Ecto.Query , warn : false alias Ecto.Changeset alias Flop alias MyApp . { Pet , Repo } @spec list_pets ( Flop . t ( ) ) :: { :ok , { [ Pet . t ( ) ] , Flop.Meta . t } } | { :error , Changeset . t ( ) } def list_pets ( flop \\\\ % Flop { } ) do Flop . validate_and_run ( Pet , flop , for : Pet ) end end The for option sets the Ecto schema for which you derived Flop.Schema . If you didn't derive Flop.Schema as described above and don't care to do so, you can omit this option (not recommended, unless you only deal with internally generated, safe parameters). On success, Flop.validate_and_run/3 returns an :ok tuple, with the second element being a tuple with the data and the meta data. { :ok , { [ % Pet { } ] , % Flop.Meta { } } } Consult the docs for more info on the Meta struct. If you prefer to validate the parameters in your controllers, you can use Flop.validate/2 or Flop.validate!/2 and Flop.run/3 instead. defmodule MyAppWeb.PetController do use MyAppWeb , :controller alias Flop alias MyApp.Pets alias MyApp.Pets.Pet action_fallback MyAppWeb.FallbackController def index ( conn , params ) do with { :ok , flop } &lt;- Flop . validate ( params , for : Pet ) do pets = Pets . list_pets ( flop ) render ( conn , &quot;index.html&quot; , pets : pets ) end end end defmodule MyApp.Pets do import Ecto.Query , warn : false alias Flop alias MyApp.Pets.Pet alias MyApp.Repo @spec list_pets ( Flop . t ( ) ) :: { [ Pet . t ( ) ] , Flop.Meta . t } def list_pets ( flop \\\\ % Flop { } ) do Flop . run ( Pet , flop , for : Pet ) end end If you only need the data, or if you only need the meta data, you can also call Flop.all/3 , Flop.meta/3 or Flop.count/3 directly. If you didn't configure a default repo as described above or if you want to override the default repo, you can pass it as an option to any function that uses the repo: Flop . validate_and_run ( Pet , flop , repo : MyApp.Repo ) Flop . all ( Pet , flop , repo : MyApp.Repo ) Flop . meta ( Pet , flop , repo : MyApp.Repo ) # etc. See the docs for more detailed information.","ref":"readme.html#usage","title":"Flop - Usage","type":"extras"},{"doc":"Flop Phoenix is a companion library that defines view helpers for use in Phoenix templates.","ref":"readme.html#flop-phoenix","title":"Flop - Flop Phoenix","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"Added Allow to define join fields in Flop.Schema . Allow to define compound fields in Flop.Schema . Support filtering by join fields. Support filtering by compound fields. New filter operator empty . New filter operator not_empty . New function Flop.set_page/2 . Changed Rename option get_cursor_value_func to cursor_value_func . Silently ignore filters with nil value for the field or the value instead of raising an ArgumentError . Allow passing a string as the second argument to Flop.push_order/2 .","ref":"changelog.html#0-12-0-2021-08-11","title":"Changelog - [0.12.0] - 2021-08-11","type":"extras"},{"doc":"Added New functions Flop.Cursor.get_cursor_from_node/2 and Flop.Cursor.get_cursor_from_edge/2 . New function Flop.get_option/2 . Support Ecto prefixes. Changed Use Flop.Cursor.get_cursor_from_node/2 as default for the :get_cursor_value_func option. Deprecate Flop.Cursor.get_cursor_from_map/2 . Use Flop.Cursor.get_cursor_from_node/2 instead. Flop.Relay.edges_from_result/2 can now handle nil instead of a map as edge information in a query result.","ref":"changelog.html#0-11-0-2021-06-13","title":"Changelog - [0.11.0] - 2021-06-13","type":"extras"},{"doc":"Added Add function Flop.push_order/2 for updating the order_by and order_directions values of a Flop struct.","ref":"changelog.html#0-10-0-2021-05-03","title":"Changelog - [0.10.0] - 2021-05-03","type":"extras"},{"doc":"Fixed Fixed type spec of Flop.Schema.default_order/1 .","ref":"changelog.html#0-9-1-2020-10-21","title":"Changelog - [0.9.1] - 2020-10-21","type":"extras"},{"doc":"Added Add like , like_and , like_or , ilike , ilike_and and ilike_or filter operators. Add option to disable pagination types globally, for a schema or locally. Add options to disable ordering or filtering. Allow global configuration of get_cursor_value_func , max_limit and default_limit . Add Flop.option type, improve documentation of available options. Add Flop.Cursor.decode!/1 . Changed Refactored the parameter validation. Default limits are now applied to all pagination types. Added validation for the after / before cursor values. Flop.Cursor.decode/1 returns :ok tuple or :error now instead of raising an error if the cursor is invalid. Flop.Cursor.decode/1 returns an error if the decoded cursor value is not a map with atom keys. Improved documentation.","ref":"changelog.html#0-9-0-2020-10-16","title":"Changelog - [0.9.0] - 2020-10-16","type":"extras"},{"doc":"Fixed Default limit was overriding first / last parameters when building query.","ref":"changelog.html#0-8-4-2020-10-14","title":"Changelog - [0.8.4] - 2020-10-14","type":"extras"},{"doc":"Fixed Cursor-based pagination: has_next_page? was set when querying with last based on before being set. Likewise, has_previous_page? was set when querying with first based on after being set. Both assumptions are wrong. In both cases, the values are always set to false now.","ref":"changelog.html#0-8-3-2020-10-14","title":"Changelog - [0.8.3] - 2020-10-14","type":"extras"},{"doc":"Changed Order directions are not restricted anymore for cursor-based pagination. Fixed Query for cursor-based pagination returned wrong results when using more than one cursor field. Query for cursor-based pagination returned wrong results when using last / before .","ref":"changelog.html#0-8-2-2020-10-08","title":"Changelog - [0.8.2] - 2020-10-08","type":"extras"},{"doc":"Changed Allow structs in cursor values.","ref":"changelog.html#0-8-1-2020-10-07","title":"Changelog - [0.8.1] - 2020-10-07","type":"extras"},{"doc":"Added Support for cursor-based pagination. Thanks to @bunker-inspector. Add functions to turn query results into Relay connection format when using cursor-based pagination.","ref":"changelog.html#0-8-0-2020-10-07","title":"Changelog - [0.8.0] - 2020-10-07","type":"extras"},{"doc":"Fixed Calculation of has_next_page? was wrong.","ref":"changelog.html#0-7-1-2020-09-04","title":"Changelog - [0.7.1] - 2020-09-04","type":"extras"},{"doc":"Added Flop.Schema now allows to set a default sort order. Changed Passing a limit without an offset will now set the offset to 0. Passing a page size without a page will now set the page to 1.","ref":"changelog.html#0-7-0-2020-08-04","title":"Changelog - [0.7.0] - 2020-08-04","type":"extras"},{"doc":"Changed Add Flop to Meta struct. Fixed Type Flop.Filter.op didn't include all operators.","ref":"changelog.html#0-6-1-2020-06-17","title":"Changelog - [0.6.1] - 2020-06-17","type":"extras"},{"doc":"Added New struct Flop.Meta . New function Flop.all/3 . New function Flop.count/3 . New function Flop.meta/3 . New function Flop.run/3 . New function Flop.validate_and_run/3 . New function Flop.validate_and_run!/3 .","ref":"changelog.html#0-6-0-2020-06-14","title":"Changelog - [0.6.0] - 2020-06-14","type":"extras"},{"doc":"Added New function Flop.validate!/2 . New filter operator :in . Fixed Filter validation was using sortable fields instead of filterable fields.","ref":"changelog.html#0-5-0-2020-05-28","title":"Changelog - [0.5.0] - 2020-05-28","type":"extras"},{"doc":"Added Added =~ filter operator. Fixed Query function wasn't generating valid where clauses for filters.","ref":"changelog.html#0-4-0-2020-05-27","title":"Changelog - [0.4.0] - 2020-05-27","type":"extras"},{"doc":"Added Added a default_limit option to Flop.Schema .","ref":"changelog.html#0-3-0-2020-05-22","title":"Changelog - [0.3.0] - 2020-05-22","type":"extras"},{"doc":"Added Added a max_limit option to Flop.Schema . When set, Flop validates that the limit and page_size parameters don't exceed the configured max limit.","ref":"changelog.html#0-2-0-2020-05-20","title":"Changelog - [0.2.0] - 2020-05-20","type":"extras"},{"doc":"initial release","ref":"changelog.html#0-1-0-2019-10-19","title":"Changelog - [0.1.0] - 2019-10-19","type":"extras"}]